@@ -9,7 +9,6 @@ NAME
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
-#define PNG_INTERNAL
 #include "png.h"
 
 #include "sng.h"
@@ -151,7 +150,7 @@ static color_item *find_by_cname(char *name)
  *
  ************************************************************************/
 
-static char token_buffer[BUFSIZ];
+static char token_buffer[16384];
 static int token_class;
 #define STRING_TOKEN	1
 #define PUNCT_TOKEN	2
@@ -469,7 +468,7 @@ static int keyword_validate(bool token_ok, char *stash)
     }
 }
 
-static void collect_data(int *pnbytes, char **pbytes)
+static void collect_data(int *pnbytes, png_byte **pbytes)
 /* collect data in either bitmap format */
 {
     /*
@@ -480,8 +479,17 @@ static void collect_data(int *pnbytes, char **pbytes)
      *    An ASCII string, doublequote-delimited.
      *
      * base64: 
-     *   One character per byte; values are as per RFC2045 base64:
-     * 0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+/
+     *   One character per byte; values 0-63 are mapped to these characters:
+     * 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/
+     *
+     * Note: This encoding is NOT compatible with RFC2045 base64.
+     * Indeed, RFC2045 uses this mapping for values 0-63:
+     * ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/
+     * There are other differences, e.g. we print no padding character '='.
+     *
+     * The advantage of our mapping is that for black and white images,
+     * we generate only '0' and '1' characters, which result in very
+     * readable output.
      *
      * hex: 
      *   Two hex digits per byte.
@@ -494,7 +502,7 @@ static void collect_data(int *pnbytes, char **pbytes)
      *
      * In either format, whitespace is ignored.
      */
-    char *bytes = xalloc(MEMORY_QUANTUM);
+    png_byte *bytes = xalloc(MEMORY_QUANTUM);
     int quanta = 1;
     int	nbytes = 0;
     int ocount = 0;
@@ -531,7 +539,7 @@ static void collect_data(int *pnbytes, char **pbytes)
 	int width = short_numeric(get_token());
 	int height = short_numeric(get_token());
 
-	if (width != info_ptr->width && height != info_ptr->height)
+	if (width != png_get_image_width(png_ptr, info_ptr) && height != png_get_image_height(png_ptr, info_ptr))
 	    fatal("pbm image dimensions don't mastch IHDR");
 	fmt = P1_FMT;
     }
@@ -542,7 +550,7 @@ static void collect_data(int *pnbytes, char **pbytes)
 
 	maxval = short_numeric(get_token());
 
-	if (width != info_ptr->width && height != info_ptr->height)
+	if (width != png_get_image_width(png_ptr, info_ptr) && height != png_get_image_height(png_ptr, info_ptr))
 	    fatal("ppm image dimensions don't match IHDR");
 	fmt = P3_FMT;
     }
@@ -585,18 +593,20 @@ static void collect_data(int *pnbytes, char **pbytes)
 	    switch(fmt)
 	    {
 	    case BASE64_FMT:
-		if (!isalpha(c) && !isdigit(c))
-		    fatal("bad character %02x in data block", c);
-		else if (isdigit(c))
+		// NOTE: The mapping below differs from base64, which uses this mapping:
+		// ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/
+		if (isdigit(c))
 		    value = c - '0';
-		else if (isupper(c))
+		else if (isalpha(c) && isupper(c))
 		    value = (c - 'A') + 10;
-		else if (islower(c))
+		else if (isalpha(c) && islower(c))
 		    value = (c - 'a') + 36;
 		else if (c == '+')
 		    value = 62;
-		else /* if (c == '/') */
+		else if (c == '/')
 		    value = 63;
+		else
+		    fatal("bad character %02x in data block", c);
 		bytes[nbytes++] = value;
 		break;
 
@@ -657,17 +667,18 @@ static void collect_data(int *pnbytes, char **pbytes)
 static void compile_IHDR(void)
 /* parse IHDR specification, set corresponding bits in info_ptr */
 {
-    int d = 0;
+    int d = 8;
+    png_uint_32 width = 0;
+    png_uint_32 height = 0;
+    int color_type = PNG_COLOR_TYPE_GRAY;
+    int interlace_type = PNG_INTERLACE_NONE;
 
     /* read IHDR data */
-    info_ptr->bit_depth = 8;
-    info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
-    info_ptr->interlace_type = PNG_INTERLACE_NONE;
     while (get_inner_token())
 	if (token_equals("height"))
-	    info_ptr->height = long_numeric(get_token());
+	    height = long_numeric(get_token());
 	else if (token_equals("width"))
-	    info_ptr->width = long_numeric(get_token());
+	    width = long_numeric(get_token());
 	else if (token_equals("bitdepth"))
 	    d = byte_numeric(get_token());
         else if (token_equals("using"))
@@ -675,33 +686,35 @@ static void compile_IHDR(void)
         else if (token_equals("grayscale"))
 	    continue;			/* so is grayscale */
         else if (token_equals("palette"))
-	    info_ptr->color_type |= PNG_COLOR_MASK_PALETTE;
+	    color_type |= PNG_COLOR_MASK_PALETTE;
         else if (token_equals("color"))
-	    info_ptr->color_type |= PNG_COLOR_MASK_COLOR;
+	    color_type |= PNG_COLOR_MASK_COLOR;
         else if (token_equals("alpha"))
-	    info_ptr->color_type |= PNG_COLOR_MASK_ALPHA;
+	    color_type |= PNG_COLOR_MASK_ALPHA;
         else if (token_equals("with"))
 	    continue;			/* `with' is just syntactic sugar */
         else if (token_equals("interlace"))
-	    info_ptr->interlace_type = PNG_INTERLACE_ADAM7;
+	    interlace_type = PNG_INTERLACE_ADAM7;
 	else
 	    fatal("bad token `%s' in IHDR specification", token_buffer);
 
     /* IHDR sanity checks */
-    if (!info_ptr->height)
+    if (!height)
 	fatal("image height is zero or nonexistent");
-    else if (!info_ptr->width)
+    else if (!width)
 	fatal("image width is zero or nonexistent");
     else if (d != 1 && d != 2 && d != 4 && d != 8 && d != 16)
 	fatal("illegal bit depth %d in IHDR", d);
-    else if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE)
+    else if (color_type == PNG_COLOR_TYPE_PALETTE)
     {
 	if (d > 8)
 	    fatal("bit depth of paletted images must be 1, 2, 4, or 8");
     }
-    else if ((info_ptr->color_type != PNG_COLOR_TYPE_GRAY) && d!=8 && d!=16)
+    else if ((color_type != PNG_COLOR_TYPE_GRAY) && d!=8 && d!=16)
 	fatal("bit depth of RGB- and alpha-using images must be 8 or 16");
-    info_ptr->bit_depth = d;
+
+    png_set_IHDR(png_ptr, info_ptr, width, height, d, color_type,
+                 interlace_type, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 }
 
 static void compile_PLTE(void)
@@ -753,8 +766,8 @@ static void compile_PLTE(void)
 static void compile_IDAT(void)
 /* parse IDAT specification and emit corresponding bits */
 {
-    int		nbits;
-    char	*bits;
+    int			nbits;
+    png_byte	*bits;
 #ifdef PNG_INFO_IMAGE_SUPPORTED
     png_unknown_chunk	chunk;
 #endif /* PNG_INFO_IMAGE_SUPPORTED */
@@ -768,10 +781,11 @@ static void compile_IDAT(void)
     png_write_chunk(png_ptr, "IDAT", bits, nbits);
     free(bits);
 #else
-    strcpy(chunk.name, "IDAT");
+    memcpy(chunk.name, "IDAT", sizeof(chunk.name));
     chunk.data = bits;
     chunk.size = nbits;
     png_set_unknown_chunks(png_ptr, info_ptr, &chunk, 1);
+// TODO: also use png_set_unknown_chunk_location if libpng before 1.6.0
     png_free(png_ptr, bits);
 #endif /* PNG_INFO_IMAGE_SUPPORTED */
 }
@@ -867,7 +881,8 @@ static void compile_iCCP(void)
 /* compile and emit an iCCP chunk */
 {
     int nname = 0, data_len = 0;
-    char name[PNG_KEYWORD_MAX_LENGTH+1], *data;
+    char name[PNG_KEYWORD_MAX_LENGTH+1];
+    png_byte *data;
 
     while (get_inner_token())
 	if (token_equals("name"))
@@ -878,7 +893,7 @@ static void compile_iCCP(void)
     if (!nname || !data_len)
 	fatal("incomplete iCCP specification");
 
-    png_set_iCCP(png_ptr, info_ptr, name, PNG_TEXT_COMPRESSION_NONE, 
+    png_set_iCCP(png_ptr, info_ptr, name, PNG_COMPRESSION_TYPE_BASE,
 		 data, data_len);
     free(data);
 }
@@ -887,8 +902,10 @@ static void compile_sBIT(void)
 /* compile an sBIT chunk, set corresponding bits in info_ptr */
 {
     png_color_8	sigbits;
-    bool color = (info_ptr->color_type & (PNG_COLOR_MASK_PALETTE | PNG_COLOR_MASK_COLOR));
-    int sample_depth = ((info_ptr->color_type & PNG_COLOR_MASK_PALETTE) ? 8 : info_ptr->bit_depth);
+    png_byte	color_type = png_get_color_type(png_ptr, info_ptr);
+    png_byte	bit_depth = png_get_bit_depth(png_ptr, info_ptr);
+    bool	color = (color_type & (PNG_COLOR_MASK_PALETTE | PNG_COLOR_MASK_COLOR));
+    int		sample_depth = ((color_type & PNG_COLOR_MASK_PALETTE) ? 8 : bit_depth);
 
     while (get_inner_token())
 	if (token_equals("red"))
@@ -925,7 +942,7 @@ static void compile_sBIT(void)
 	}
 	else if (token_equals("alpha"))
 	{
-	    if (info_ptr->color_type & PNG_COLOR_MASK_ALPHA)
+	    if (color_type & PNG_COLOR_MASK_ALPHA)
 		fatal("No alpha channel in this image type");
 	    sigbits.alpha = byte_numeric(get_token());
 	    if (sigbits.alpha > sample_depth)
@@ -942,35 +959,36 @@ static void compile_bKGD(void)
 /* compile a bKGD chunk, put data in info structure */
 {
     png_color_16	bkgbits;
+    png_byte		color_type = png_get_color_type(png_ptr, info_ptr);
 
     while (get_inner_token())
 	if (token_equals("red"))
 	{
-	    if (!(info_ptr->color_type & PNG_COLOR_MASK_COLOR))
+	    if (!(color_type & PNG_COLOR_MASK_COLOR))
 		fatal("Can't use color background with this image type");
 	    bkgbits.red = short_numeric(get_token());
 	}
 	else if (token_equals("green"))
 	{
-	    if (!(info_ptr->color_type & PNG_COLOR_MASK_COLOR))
+	    if (!(color_type & PNG_COLOR_MASK_COLOR))
 		fatal("Can't use color background with this image type");
 	    bkgbits.green = short_numeric(get_token());
 	}
 	else if (token_equals("blue"))
 	{
-	    if (!(info_ptr->color_type & PNG_COLOR_MASK_COLOR))
+	    if (!(color_type & PNG_COLOR_MASK_COLOR))
 		fatal("Can't use color background with this image type");
 	    bkgbits.blue = short_numeric(get_token());
 	}
 	else if (token_equals("gray"))
 	{
-	    if (info_ptr->color_type & (PNG_COLOR_MASK_COLOR | PNG_COLOR_MASK_PALETTE))
+	    if (color_type & (PNG_COLOR_MASK_COLOR | PNG_COLOR_MASK_PALETTE))
 		fatal("Can't use color background with this image type");
 	    bkgbits.gray = short_numeric(get_token());
 	}
 	else if (token_equals("index"))
 	{
-	    if (!(info_ptr->color_type & PNG_COLOR_MASK_PALETTE))
+	    if (!(color_type & PNG_COLOR_MASK_PALETTE))
 		fatal("Can't use index background with a non-palette image");
 	    bkgbits.index = byte_numeric(get_token());
 	}
@@ -986,18 +1004,17 @@ static void compile_hIST(void)
 {
     png_uint_16	hist[256];
     int		nhist = 0;
+    png_colorp	palette;
+    int		num_palette;
+
+    png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette);
 
     while (get_inner_token())
 	/* comma */
 	hist[nhist++] = short_numeric(TRUE);
 
-#ifndef MNG_INTERFACE
-    if (nhist != info_ptr->num_palette)
-	fatal("number of hIST values (%d) for palette doesn't match palette size (%d)", nhist, info_ptr->num_palette);
-#else
-    if (nhist != info_ptr->palette.size)
-	fatal("number of hIST values (%d) for palette doesn't match palette size (%d)", nhist, info_ptr->palette.size);
-#endif /* GUG */
+    if (nhist != num_palette)
+	fatal("number of hIST values (%d) for palette doesn't match palette size (%d)", nhist, num_palette);
 
     png_set_hIST(png_ptr, info_ptr, hist);
 }
@@ -1007,11 +1024,13 @@ static void compile_tRNS(void)
 {
     png_byte	trans[256];
     int		ntrans = 0;
+    png_byte	color_type;
     png_color_16	tRNSbits;
 
     memset(&tRNSbits, '0', sizeof(tRNSbits));
+    color_type = png_get_color_type(png_ptr, info_ptr);
 
-    switch (info_ptr->color_type)
+    switch (color_type)
     {
     case PNG_COLOR_TYPE_GRAY:
 	require_or_die("gray");
@@ -1216,7 +1235,7 @@ static void compile_iTXt(void)
 {
     char	language[PNG_KEYWORD_MAX_LENGTH+1];
     char	keyword[PNG_KEYWORD_MAX_LENGTH+1]; 
-    char	transkey[PNG_KEYWORD_MAX_LENGTH+1]; 
+    char	transkey[PNG_STRING_MAX_LENGTH+1]; 
     char	text[PNG_STRING_MAX_LENGTH+1];
     int		nlanguage = 0, nkeyword = 0, ntranskey = 0, ntext = 0;
     int		compression = PNG_TEXT_COMPRESSION_NONE;
@@ -1229,7 +1248,7 @@ static void compile_iTXt(void)
 	else if (token_equals("keyword"))
 	    nkeyword = keyword_validate(get_token(), keyword);
  	else if (token_equals("translated"))
-	    ntranskey = keyword_validate(get_token(), transkey);
+	    ntranskey = string_validate(get_token(), transkey);
 	else if (token_equals("text"))
 	    ntext = string_validate(get_token(), text);
 	else if (token_equals("compressed"))
@@ -1237,7 +1256,7 @@ static void compile_iTXt(void)
 	else
 	    fatal("bad token `%s' in iTXt specification", token_buffer);
 
-    if (!language || !keyword || !transkey || !text)
+    if (!nlanguage || !nkeyword || !ntranskey || !ntext)
 	fatal("keyword or text is missing");
 
     textblk.key = keyword;
@@ -1447,7 +1466,7 @@ static void compile_sCAL(void)
     double	width = 0, height = 0;
     int 	nunit = 0;
     png_byte	unitbyte = 0;
-#if !defined(FLOATING_POINT_SUPPORTED) && defined(FIXED_POINT_SUPPORTED)
+#if !defined(PNG_FLOATING_POINT_SUPPORTED) && defined(PNG_FIXED_POINT_SUPPORTED)
     char	width_s[BUFSIZ], height_s[BUFSIZ];
 #endif
 
@@ -1465,14 +1484,14 @@ static void compile_sCAL(void)
 	else if (token_equals("width"))
 	{
 	    width = double_numeric(get_token());
-#if !defined(FLOATING_POINT_SUPPORTED) && defined(FIXED_POINT_SUPPORTED)
+#if !defined(PNG_FLOATING_POINT_SUPPORTED) && defined(PNG_FIXED_POINT_SUPPORTED)
 	    strcpy(width_s, token_buffer);
 #endif
 	}
 	else if (token_equals("height"))
 	{
 	    height = double_numeric(get_token());
-#if !defined(FLOATING_POINT_SUPPORTED) && defined(FIXED_POINT_SUPPORTED)
+#if !defined(PNG_FLOATING_POINT_SUPPORTED) && defined(PNG_FIXED_POINT_SUPPORTED)
 	    strcpy(height_s, token_buffer);
 #endif
 	}
@@ -1499,9 +1518,11 @@ static void compile_gIFg(void)
 
     memset(&chunk, '\0', sizeof(chunk));
     memset(chunkdata, '\0', sizeof(chunkdata));
-    strcpy(chunk.name, "gIFg");
+    memcpy(chunk.name, "gIFg", sizeof(chunk.name));
     chunk.data = chunkdata;
     chunk.size = 4;
+//    chunk.location = TODO; PNG_HAVE_IHDR or PNG_HAVE_PLTE or PNG_AFTER_IDAT
+    chunk.location = PNG_HAVE_IHDR; // FIXME
 
     while (get_inner_token())
 	if (token_equals("disposal"))
@@ -1518,18 +1539,22 @@ static void compile_gIFg(void)
 	    fatal("invalid token `%s' in gIFg specification", token_buffer);
 
     png_set_unknown_chunks(png_ptr, info_ptr, &chunk, 1);
+// TODO: also use png_set_unknown_chunk_location if libpng before 1.6.0
 }
 
 static void compile_gIFx(void)
 /* parse gIFx specification and queue up the corresponding chunk */
 {
-    png_byte chunkdata[PNG_STRING_MAX_LENGTH], buf[PNG_STRING_MAX_LENGTH];
+    png_byte chunkdata[PNG_STRING_MAX_LENGTH];
+    char buf[PNG_STRING_MAX_LENGTH];
     png_unknown_chunk chunk;
 
     memset(&chunk, '\0', sizeof(chunk));
     memset(chunkdata, '\0', sizeof(chunkdata));
-    strcpy(chunk.name, "gIFx");
+    memcpy(chunk.name, "gIFx", sizeof(chunk.name));
     chunk.data = chunkdata;
+//    chunk.location = TODO; PNG_HAVE_IHDR or PNG_HAVE_PLTE or PNG_AFTER_IDAT
+    chunk.location = PNG_HAVE_IHDR; // FIXME
 
     while (get_inner_token())
 	if (token_equals("identifier"))
@@ -1549,7 +1574,7 @@ static void compile_gIFx(void)
 	else if (token_equals("data"))
 	{
 	    int datalen;
-	    char *data;
+	    png_byte *data;
 
 	    collect_data(&datalen, &data);
 	    memcpy(chunkdata + 11, data, datalen);
@@ -1558,17 +1583,23 @@ static void compile_gIFx(void)
 	else
 	    fatal("invalid token `%s' in gIFx specification", token_buffer);
 
-    chunk.size = 11 + strlen(chunkdata + 11);
+    chunk.size = 11 + strlen((char *)chunkdata + 11);
 
     png_set_unknown_chunks(png_ptr, info_ptr, &chunk, 1);
+// TODO: also use png_set_unknown_chunk_location if libpng before 1.6.0
 }
 
 static void compile_IMAGE(void)
 /* parse IMAGE specification and emit corresponding bits */
 {
     int		i, nbytes, bytes_per_sample = 0, nsamples, input_width;
-    char	*bytes;
-    int		doublewidth = info_ptr->bit_depth == 16 ? 2 : 1;
+    png_byte	*bytes;
+    png_byte	color_type = png_get_color_type(png_ptr, info_ptr);
+    png_byte	bit_depth = png_get_bit_depth(png_ptr, info_ptr);
+    int		doublewidth = bit_depth == 16 ? 2 : 1;
+    png_bytepp	row_pointers = 0;
+    int		width = png_get_image_width(png_ptr, info_ptr);
+    int		height = png_get_image_height(png_ptr, info_ptr);
 
     write_transform_options = 0;
     while (get_inner_token())
@@ -1601,7 +1632,7 @@ static void compile_IMAGE(void)
 	    fatal("invalid token `%s' in IMAGE specification", token_buffer);
 
     /* compute input sample size in bits */
-    switch (info_ptr->color_type)
+    switch (color_type)
     {
     case PNG_COLOR_TYPE_GRAY:
 	bytes_per_sample = doublewidth;
@@ -1630,12 +1661,12 @@ static void compile_IMAGE(void)
     /*
      * Compute the actual size of the image in samples.
      */
-    input_width = nbytes / info_ptr->height;
-    if (info_ptr->bit_depth >= 8)
+    input_width = nbytes / height;
+    if (bit_depth >= 8)
 	nsamples = nbytes / bytes_per_sample;
     else
     {
-	int samples_per_byte = (8 / info_ptr->bit_depth);
+	int samples_per_byte = (8 / bit_depth);
 
 	nsamples = nbytes * samples_per_byte;
 
@@ -1644,17 +1675,17 @@ static void compile_IMAGE(void)
 	 * byte corresponding to the last pixel(s) in a row when the
 	 * width is not a multiple of 8.  Correct for this.
 	 */
-	if (info_ptr->width % 8)
+	if (width % 8)
 	{
-	    int excess_samples_per_line = (info_ptr->width % samples_per_byte);
+	    int excess_samples_per_line = (width % samples_per_byte);
 
 	    if (excess_samples_per_line)
-		nsamples -= info_ptr->height * (samples_per_byte - excess_samples_per_line);
+		nsamples -= height * (samples_per_byte - excess_samples_per_line);
 	}
     }
-    if (nsamples != info_ptr->width * info_ptr->height)
+    if (nsamples != width * height)
 	fatal("sample count (%d) doesn't match width*height (%d*%d) in IHDR",
-	      nsamples, info_ptr->width, info_ptr->height);
+	      nsamples, width, height);
 
 #ifdef PNG_DEBUG
 #if (PNG_DEBUG >= 6)
@@ -1673,19 +1704,18 @@ static void compile_IMAGE(void)
 #endif
 #endif
 
+    row_pointers = (png_byte **)xalloc(sizeof(png_bytep) * height);
+    for (i = 0; i < height; i++)
+	row_pointers[i] = &bytes[i * input_width];
+
 #ifndef PNG_INFO_IMAGE_SUPPORTED
     /* got the bits; now write them out */
-    row_pointers = (png_byte **)xalloc(sizeof(char *) * info_ptr->height);
-    for (i = 0; i < info_ptr->height; i++)
-	row_pointers[i] = &bytes[i * input_width];
     png_write_image(png_ptr, row_pointers);
     free(bytes);
+    free(row_pointers);
 #else
     /* got the bits; attach them to the info structure */
-    info_ptr->row_pointers = (png_byte **)xalloc(sizeof(char *) * info_ptr->height);
-    for (i = 0; i < info_ptr->height; i++)
-	info_ptr->row_pointers[i] = &bytes[i * input_width];
-    info_ptr->valid |= PNG_INFO_IDAT;
+    png_set_rows(png_ptr, info_ptr, row_pointers);
 #endif /* PNG_INFO_IMAGE_SUPPORTED */
 }
 
@@ -1693,13 +1723,13 @@ static void compile_private(char *name)
 /* compile a private chunk */
 {
     int			nbytes;
-    char		*bytes;
+    png_byte		*bytes;
     png_unknown_chunk	chunk;
 
     if (strlen(name) != 4)
 	fatal("wrong length for chunk name %s", token_buffer);
     else
-	strcpy(chunk.name, name);
+	memcpy(chunk.name, name, sizeof(chunk.name));
 
     require_or_die("{");
     collect_data(&nbytes, &bytes);
@@ -1707,7 +1737,10 @@ static void compile_private(char *name)
 
     chunk.data = bytes;
     chunk.size = nbytes;
+//    chunk.location = TODO; PNG_HAVE_IHDR or PNG_HAVE_PLTE or PNG_AFTER_IDAT
+    chunk.location = PNG_HAVE_IHDR; // FIXME
     png_set_unknown_chunks(png_ptr, info_ptr, &chunk, 1);
+// TODO: also use png_set_unknown_chunk_location if libpng before 1.6.0
     png_free(png_ptr, bytes);
 }
 
@@ -1769,6 +1802,12 @@ int sngc(FILE *fin, char *name, FILE *fout)
 
     write_transform_options = PNG_TRANSFORM_IDENTITY;
 
+    /* initialize per-input-file chunk properties */
+    for (chunkprops *pp = properties;
+             pp < properties + sizeof(properties)/sizeof(chunkprops);
+             pp++)
+        pp->count = 0;
+
     /* interpret the following chunk specifications */
     prevchunk = NONE;
     while (get_token())
@@ -1805,7 +1844,7 @@ int sngc(FILE *fin, char *name, FILE *fout)
 		fatal("PLTE chunk encountered after bKGD");
 	    else if (properties[tRNS].count)
 		fatal("PLTE chunk encountered after tRNS");
-	    else if (!(info_ptr->color_type & PNG_COLOR_MASK_PALETTE))
+	    else if (!(png_get_color_type(png_ptr, info_ptr) & PNG_COLOR_MASK_PALETTE))
 		fatal("PLTE chunk specified for non-palette image type");
 #ifndef PNG_INFO_IMAGE_SUPPORTED
 	    png_write_info_before_PLTE(png_ptr, info_ptr);
@@ -1964,7 +2003,7 @@ int sngc(FILE *fin, char *name, FILE *fout)
 
     /* end-of-file sanity checks */
     linenum = EOF;
-    if (!properties[PLTE].count && (info_ptr->color_type & PNG_COLOR_MASK_PALETTE))
+    if (!properties[PLTE].count && (png_get_color_type(png_ptr, info_ptr) & PNG_COLOR_MASK_PALETTE))
 	fatal("palette property set, but no PLTE chunk found");
     if (!properties[IDAT].count)
 	fatal("no image data");
